package com.fastbee.ggroup.service.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.fastbee.common.exception.ServiceException;
import com.fastbee.common.utils.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.fastbee.ggroup.mapper.GLegendMapper;
import com.fastbee.ggroup.domain.GLegend;
import com.fastbee.ggroup.service.IGLegendService;

/**
 * 图例Service业务层处理
 *
 * @author kerwincui
 * &#064;date  2024-10-08
 */
@Service
@Slf4j
public class GLegendServiceImpl implements IGLegendService
{
    @Autowired
    private GLegendMapper gLegendMapper;


    /**
     * 查询图例
     *
     * @param id 图例主键
     * @return 图例
     */
    @Override
    public GLegend selectGLegendById(Long id)
    {
        return gLegendMapper.selectOne(new LambdaQueryWrapper<GLegend>()
                .select(GLegend::getId,GLegend::getIcon,GLegend::getName,GLegend::getCategory)
                .eq(GLegend::getId,id));
    }

    /**
     * 查询/搜索图例列表
     * @param gLegend 图例
     * @return 图例
     */
    @Override
    public List<GLegend> selectGLegendList(GLegend gLegend) {
       return new LambdaQueryChainWrapper<>(gLegendMapper)
                .select(GLegend::getId,GLegend::getIcon,GLegend::getName,GLegend::getCategory)
                .like(StringUtils.isNotBlank(gLegend.getSearchValue()), GLegend::getName, gLegend.getSearchValue())
                .list();
    }

    /**
     * 新增图例
     *
     * @param gLegend 图例
     * @return 结果
     */
    @Override
    public int insertGLegend(GLegend gLegend)
    {
        //图例名称不能重复
        if(!new LambdaQueryChainWrapper<>(gLegendMapper)
                .select(GLegend::getId,GLegend::getName)
                .eq(GLegend::getName,gLegend.getName().trim()).list().isEmpty()){
            throw new ServiceException("图例名称已存在!");
        }
        return gLegendMapper.insertGLegend(gLegend);
    }

    /**
     * 修改图例
     *
     * @param gLegend 图例
     * @return 结果
     */
    @Override
    public int updateGLegend(GLegend gLegend)
    {
        //图例名称不能重复
        if(!new LambdaQueryChainWrapper<>(gLegendMapper)
                .select(GLegend::getId,GLegend::getName)
                .eq(GLegend::getName,gLegend.getName().trim()).ne(GLegend::getId,gLegend.getId()).list().isEmpty()){
            throw new ServiceException("图例名称已存在!");
        }
        gLegend.setUpdateTime(DateUtils.getNowDate());
        return gLegendMapper.updateGLegend(gLegend);
    }
    /**
     * 批量删除图例
     *
     * @param ids 需要删除的图例主键
     * @return 结果
     */
    @Override
    public int deleteGLegendByIds(Long[] ids)
    {
        return gLegendMapper.deleteGLegendByIds(ids);
    }

    /**
     * 删除图例信息
     *
     * @param id 图例主键
     * @return 结果
     */
    @Override
    public int deleteGLegendById(Long id)
    {
        return gLegendMapper.deleteGLegendById(id);
    }

    /**
     * 按照类别返回所有类别下图例列表
     * @return 图例列表
     */
    @Override
    public List<Map<String, Object>> getLegendListByCategory() {
        return new LambdaQueryChainWrapper<>(gLegendMapper)
                .select(GLegend::getId,GLegend::getIcon, GLegend::getName,GLegend::getCategory)
                .list().stream()
                .collect(Collectors.groupingBy(GLegend::getCategory))
                .entrySet().stream()
                .map(entry -> {
                            Map<String, Object> itemMap = new HashMap<>();
                            itemMap.put("category", entry.getKey());
                            itemMap.put("list", entry.getValue());
                            return itemMap;
                        }
                ).collect((Collectors.toList()));
    }
}
