package com.fastbee.ggroup.service.impl;

import java.io.*;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;


import cn.hutool.http.HttpUtil;
import cn.hutool.json.JSONException;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fastbee.common.config.RuoYiConfig;
import com.fastbee.common.exception.ServiceException;
import com.fastbee.ggroup.domain.GSiteGroups;
import com.fastbee.ggroup.domain.dto.GGroupSiteDto;
import com.fastbee.ggroup.domain.dto.GGroupSiteRelateDto;
import com.fastbee.ggroup.domain.dto.GSitesEditDto;
import com.fastbee.ggroup.domain.dto.GSitesSelectDto;
import com.fastbee.ggroup.mapper.GSiteGroupsMapper;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.IOUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.fastbee.ggroup.mapper.GSitesMapper;
import com.fastbee.ggroup.domain.GSites;
import com.fastbee.ggroup.service.IGSitesService;
import org.springframework.transaction.annotation.Transactional;

/**
 * 站点Service业务层处理
 * 
 * @author kerwincui
 * &#064;date  2024-09-29
 */
@Service
@Slf4j
public class GSitesServiceImpl extends ServiceImpl<GSiteGroupsMapper,GSiteGroups> implements IGSitesService {
    @Autowired
    private GSitesMapper gSitesMapper;

    @Autowired
    private GSiteGroupsMapper gSiteGroupsMapper;

    /**
     * 查询站点
     *
     * @param id 站点主键
     * @return 站点
     */
    @Override
    public GSites selectGSitesById(Long id) {
        return gSitesMapper.selectGSitesById(id);
    }

    /**
     * 查询/搜索全部站点列表
     *
     * @param gSites 站点
     * @return 站点
     */
    @Override
    public List<?> selectGSitesList(GSitesSelectDto gSites) {
        return gSitesMapper.selectList(new LambdaQueryWrapper<GSites>()
                .select(GSites::getId, GSites::getName, GSites::getIcon, GSites::getType)
                .eq(GSites::getProjectId, gSites.getProjectId())
        );
    }

    /**
     * 新增站点
     *
     * @param gGroupSiteDto 站点
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int insertGSites(GGroupSiteDto gGroupSiteDto) {

        GSites gSites = new GSites();
        BeanUtils.copyProperties(gGroupSiteDto, gSites);
        //是否存在同名站点
        GSites sameNameSite = gSitesMapper.selectOne(new LambdaQueryWrapper<GSites>()
                .select(GSites::getName, GSites::getId)
                .eq(GSites::getName, gGroupSiteDto.getName().trim()));
        if (Objects.nonNull(sameNameSite)) {
            throw new ServiceException("站点名称重复！");
        }
        //未上传站点地图json数据
        if(StringUtils.isBlank(gGroupSiteDto.getIcon())){
            throw new ServiceException("请上传站点地图json数据！");
        }
        //设置坐标值等信息
        getReadJsonFileContent(gSites);
//        插入站点表
        int inserted = gSitesMapper.insert(gSites);
        gGroupSiteDto.setSiteId(gSites.getId());
        //插入关系表
        int inserted1 = gSiteGroupsMapper.insertGSiteGroups(gGroupSiteDto);
        return inserted == 1 && inserted1 == 1 ? 1 : 0;
//            return 1;
    }

    /**
     * 读取上传的站点图信息并赋值
     * @param gSites 站点
     */
    private void getReadJsonFileContent(GSites gSites) {
        String json;
        //TODO 路径优化
        //发起http请求获取json数据
        String space = gSites.getSpace();
        if(Objects.isNull(space)){
            throw new ServiceException("请上传站点地图json数据！");
        }
        String jsonData = HttpUtil.get(space);
        System.err.println("获取到json:"+jsonData);

        //读取文件内容
        if (Objects.isNull(jsonData)) {
            throw new ServiceException("上传站点地图json数据为空！");
        }
        //校验json格式是否正确
        if (!isValidJson(jsonData)) {
            throw new ServiceException("json格式不正确！");
        }
        int startIndex = space.indexOf("/profile/upload/");
//        System.err.println("路径匹配截取开始下标:"+startIndex);
        String relativeUrl = space.substring(startIndex);
//        System.err.println("相对路径:"+relativeUrl);
        gSites.setSpace(relativeUrl);
        gSites.setSpaceValue(jsonData);

    }


    /**
     * 修改站点
     *
     * @param gSitesEditDto 站点
     * @return 结果
     */
    @Override
    public int updateGSites(GSitesEditDto gSitesEditDto) {
        //是否存在同名站点
        GSites sameNameSite = gSitesMapper.selectOne(new LambdaQueryWrapper<GSites>()
                .select(GSites::getName, GSites::getId)
                .eq(GSites::getName, gSitesEditDto.getName().trim())
                .ne(GSites::getId, gSitesEditDto.getId()));
        if (Objects.nonNull(sameNameSite)) {
            throw new ServiceException("站点名称重复！");
        }
        GSites sites = new GSites();
        BeanUtils.copyProperties(gSitesEditDto, sites);
        //读取文件内容
        getReadJsonFileContent(sites);
        return gSitesMapper.updateById(sites);
    }


    /**
     * 验证字符串是否是json格式
     */
    public  boolean isValidJson(String jsonStr) {
        try {
            JSONUtil.parseObj(jsonStr);
        } catch (JSONException e) {
            return false;
        }
        return true;
    }
    /**
     * 批量删除站点
     *
     * @param ids 需要删除的站点主键
     * @return 结果
     */
    @Override
    public int deleteGSitesByIds(Long[] ids) {
        return gSitesMapper.deleteGSitesByIds(ids);
    }

    /**
     * 删除站点信息
     *
     * @param id 站点主键
     * @return 结果
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public int deleteGSitesById(Long id) {
        return gSiteGroupsMapper.deleteById(id);
    }

    /**
     * 站点关联组
     * @param gGroupSiteRelateDto 传输对象
     * @return 结果
     */
    @Override
    public int relateGroup(GGroupSiteRelateDto gGroupSiteRelateDto) {
        return super.saveBatch(gGroupSiteRelateDto.getSiteIds()
                .stream()
                .map(siteId -> {
                    GSiteGroups gSiteGroups = new GSiteGroups();
                    gSiteGroups.setSiteId(siteId);
                    return gSiteGroups;
                })
                .collect(Collectors.toList())
                .stream()
                .peek(item -> {
                    item.setParentId(gGroupSiteRelateDto.getParentId());
                    item.setProjectId(gGroupSiteRelateDto.getProjectId());
                })
                .collect(Collectors.toList()))? 1 : 0;
    }

    @Override
    public List<GSites> searchGSitesList(GSites gSites) {
        return gSitesMapper.selectList(new LambdaQueryWrapper<GSites>()
                        .eq(GSites::getProjectId, gSites.getProjectId())
                        .select(GSites::getId, GSites::getName, GSites::getType, GSites::getIcon)
                        .like(GSites::getName, gSites.getSearchValue()));
    }



}

